import base64
import requests
import time
import logging

from harambot.config import settings
from discord import Embed

YAHOO_API_URL = "https://api.login.yahoo.com/oauth2/"
YAHOO_AUTH_URI = "request_auth?redirect_uri=oob&response_type=code&client_id="

logger = logging.getLogger("discord.harambot.utils")


def yahoo_auth(code):
    encoded_creds = base64.b64encode(
        ("{0}:{1}".format(settings.yahoo_key, settings.yahoo_secret)).encode(
            "utf-8"
        )
    )
    response = requests.post(
        url="{}get_token".format(YAHOO_API_URL),
        data={
            "code": code,
            "redirect_uri": "oob",
            "grant_type": "authorization_code",
        },
        headers={
            "Authorization": "Basic {0}".format(encoded_creds.decode("utf-8")),
            "Content-Type": "application/x-www-form-urlencoded",
        },
    )
    if response.status_code != 200:
        logger.error(
            "Failed to authenticate with Yahoo API: {}".format(response.json)
        )
        return {}
    details = response.json()

    details["token_time"] = time.time()
    return details


def create_add_embed(transaction):
    embed = Embed(title="Player Added")
    add_player_fields_to_embed(embed, transaction["players"]["0"]["player"][0])
    embed.add_field(
        name="Owner",
        value=transaction["players"]["0"]["player"][1]["transaction_data"][0][
            "destination_team_name"
        ],
    )
    if "faab_bid" in transaction:
        embed.add_field(
            name="Bid", value=transaction["faab_bid"], inline=False
        )
    return embed


def create_drop_embed(transaction):
    embed = Embed(title="Player Dropped")
    add_player_fields_to_embed(embed, transaction["players"]["0"]["player"][0])
    embed.add_field(
        name="Owner",
        value=transaction["players"]["0"]["player"][1]["transaction_data"][
            "source_team_name"
        ],
    )
    return embed


def create_add_drop_embed(transaction):
    embed = Embed(title="Player Added/ Player Dropped")
    embed.add_field(
        name="Owner",
        value=transaction["players"]["0"]["player"][1]["transaction_data"][0][
            "destination_team_name"
        ],
    )
    if "faab_bid" in transaction:
        embed.add_field(
            name="Bid", value=transaction["faab_bid"], inline=False
        )
    embed.add_field(
        name="Player Added", value="=====================", inline=False
    )
    add_player_fields_to_embed(embed, transaction["players"]["0"]["player"][0])
    embed.add_field(
        name="Player Dropped", value="=====================", inline=False
    )
    add_player_fields_to_embed(embed, transaction["players"]["1"]["player"][0])
    return embed


def add_player_fields_to_embed(embed, player):
    embed.add_field(
        name="Player", value=player[2]["name"]["full"], inline=True
    )
    embed.add_field(
        name="Team", value=player[3]["editorial_team_abbr"], inline=True
    )
    embed.add_field(
        name="Position", value=player[4]["display_position"], inline=True
    )


def get_avatar_bytes():
    # return the image from the settings.webhook_avatar_url variable as bytes
    return requests.get(settings.webhook_avatar_url).content



import discord
import logging

from harambot.config import settings
from harambot.utils import YAHOO_API_URL, YAHOO_AUTH_URI, get_avatar_bytes
from harambot.ui.modals import ConfigModal
from harambot.database.models import Guild

logger = logging.getLogger("discord.harambot.views")
logger.setLevel(logging.INFO)


class YahooAuthButton(discord.ui.Button):
    def __init__(self):
        super().__init__(
            style=discord.ButtonStyle.link,
            label="Login to Yahoo",
            url=f"{YAHOO_API_URL}{YAHOO_AUTH_URI}{settings.yahoo_key}",
        )


class ConfigGuildButton(discord.ui.Button):

    parent_view: None

    def __init__(self, parent_view: discord.ui.View):
        super().__init__(
            label="Configure Guild", style=discord.ButtonStyle.blurple
        )
        self.parent_view = parent_view

    async def callback(self, interaction: discord.Interaction):
        await interaction.response.send_modal(
            ConfigModal(
                guild_id=str(interaction.guild_id), view=self.parent_view
            )
        )


class ResetButton(discord.ui.Button):
    def __init__(self):
        super().__init__(style=discord.ButtonStyle.danger, label="Reset")

    async def callback(self, interaction: discord.Interaction):
        if (
            Guild.select()
            .where(Guild.guild_id == str(interaction.guild.id))
            .exists()
        ):
            guild = Guild.get(Guild.guild_id == str(interaction.guild.id))
            guild.delete_instance()
            await interaction.response.send_message(
                "Guild configuration reset!"
            )
        else:
            await interaction.response.send_message("Guild not configured!")


class ConfigView(discord.ui.View):
    def __init__(
        self,
    ):
        super().__init__()
        self.add_item(YahooAuthButton())
        self.add_item(ConfigGuildButton(parent_view=self))
        self.add_item(ResetButton())


class ReportConfigView(discord.ui.View):
    def __init__(self):
        super().__init__()

    @discord.ui.select(
        cls=discord.ui.ChannelSelect, channel_types=[discord.ChannelType.text]
    )
    async def select_channels(
        self,
        interaction: discord.Interaction,
        select: discord.ui.ChannelSelect,
    ):

        guild = Guild.get(Guild.guild_id == str(interaction.guild.id))

        channel = select.values[0].resolve()
        if not channel:
            channel = select.values[0].fetch()

        for webhook in await channel.webhooks():
            if webhook.user == interaction.guild.me:
                guild.transaction_polling_webhook = webhook.url
                guild.save()
                return await interaction.response.send_message(
                    f"Reports configured to go to {select.values[0].mention}"
                )

        if guild.transaction_polling_service_enabled == 0:
            guild.transaction_polling_service_enabled = 1
        else:
            try:
                webhook = discord.SyncWebhook.from_url(
                    guild.transaction_polling_webhook
                )
                webhook.delete()
            except discord.errors.NotFound:
                logger.info("Webhook not found")
            guild.transaction_polling_webhook = None

        if not guild.transaction_polling_webhook:
            webhook = await channel.create_webhook(
                name="Harambot Reports", avatar=get_avatar_bytes()
            )
            guild.transaction_polling_webhook = webhook.url

        guild.save()

        return await interaction.response.send_message(
            f"Reports configured to go to {select.values[0].mention}"
        )



import discord
import logging


from discord.ext import commands
from discord import app_commands
from typing import List, Optional
from datetime import datetime, timedelta

from harambot.yahoo_api import Yahoo
from harambot import utils

logging.setLoggerClass(logging.Logger)
logging.getLogger("yahoo_oauth").setLevel("INFO")

logger = logging.getLogger("discord.harambot.cogs.yahoo")


class YahooCog(commands.Cog):

    error_message = (
        "I'm having trouble getting that right now please try again later"
    )

    def __init__(self, bot, KEY, SECRET):
        self.bot = bot
        self.KEY = KEY
        self.SECRET = SECRET
        self.yahoo_api = Yahoo()

    @app_commands.command(
        name="standings",
        description="Returns the current standings of your league",
    )
    async def standings(self, interaction: discord.Interaction):
        logger.info("Command:Standings called in %i", interaction.guild_id)
        embed = discord.Embed(
            title="Standings",
            description="Team Name\n W-L-T",
            color=0xEEE657,
        )
        standings = self.yahoo_api.get_standings(guild_id=interaction.guild_id)
        if standings:
            for team in standings:
                embed.add_field(
                    name=team["place"],
                    value=team["record"],
                    inline=False,
                )
            await interaction.response.send_message(embed=embed)
        else:
            await interaction.response.send_message(self.error_message)

    async def roster_autocomplete(
        self,
        interaction: discord.Interaction,
        current: str,
    ) -> List[app_commands.Choice[str]]:
        teams = self.yahoo_api.get_teams(guild_id=interaction.guild_id)
        if teams:
            options = list(
                map(
                    lambda x: app_commands.Choice(
                        name=teams[x]["name"], value=teams[x]["name"]
                    ),
                    teams,
                )
            )
            return options
        return []

    @app_commands.command(
        name="roster", description="Returns the roster of the given team"
    )
    @app_commands.autocomplete(team_name=roster_autocomplete)
    async def roster(self, interaction: discord.Interaction, team_name: str):
        logger.info(
            "Command:Roster called in %i with team_name:%s",
            interaction.guild_id,
            team_name,
        )
        embed = discord.Embed(
            title="{}'s Roster".format(team_name),
            description="",
            color=0xEEE657,
        )
        roster = self.yahoo_api.get_roster(
            guild_id=interaction.guild_id, team_name=team_name
        )
        if roster:
            for player in roster:
                embed.add_field(
                    name=player["selected_position"],
                    value=player["name"],
                    inline=False,
                )
            await interaction.response.send_message(embed=embed)
        else:
            await interaction.response.send_message(self.error_message)

    @app_commands.command(
        name="trade",
        description="Create poll for latest trade for league approval",
    )
    async def trade(self, interaction: discord.Interaction):
        logger.info("Command:Trade called in %i", interaction.guild_id)
        latest_trade = self.yahoo_api.get_latest_trade(
            guild_id=interaction.guild_id
        )
        if latest_trade is None:
            await interaction.response.send_message(
                "No trades up for approval at this time"
            )
            return

        teams = self.yahoo_api.get_teams()
        if teams is None:
            await interaction.response.send_message(self.error_message)
            return
        trader = teams[latest_trade["trader_team_key"]]
        tradee = teams[latest_trade["tradee_team_key"]]
        managers = [trader["name"], tradee["name"]]

        player_set0 = []
        player_set0_details = ""
        for player in latest_trade["trader_players"]:
            if player:
                player_set0.append(player["name"])
                api_details = (
                    self.get_player_text(
                        self.yahoo_api.get_player_details(player["name"])
                    )
                    + "\n"
                )
                if api_details:
                    player_set0_details = player_set0_details + api_details
                else:
                    await interaction.send(self.error_message)
                    return

        player_set1 = []
        player_set1_details = ""
        for player in latest_trade["tradee_players"]:
            player_set1.append(player["name"])
            player_details = self.yahoo_api.get_player_details(player["name"])
            if player_details is None:
                await interaction.send(self.error_message)
                return
            api_details = self.get_player_text(player_details) + "\n"
            if api_details:
                player_set1_details = player_set1_details + api_details
            else:
                await interaction.response.send_message(self.error_message)
                return

            confirm_trade_message = "{} sends {} to {} for {}".format(
                managers[0],
                ", ".join(player_set0),
                managers[1],
                ", ".join(player_set1),
            )
            announcement = "There's collusion afoot!\n"
            embed = discord.Embed(
                title="The following trade is up for approval:",
                description=confirm_trade_message,
                color=0xEEE657,
            )
            embed.add_field(
                name="{} sends:".format(managers[0]),
                value=player_set0_details,
                inline=False,
            )
            embed.add_field(
                name="to {} for:".format(managers[1]),
                value=player_set1_details,
                inline=False,
            )
            embed.add_field(
                name="Voting",
                value=" Click :white_check_mark: for yes,\
                     :no_entry_sign: for no",
            )
            await interaction.response.send_message(
                content=announcement, embed=embed
            )
            response_message = await interaction.original_response()
            yes_emoji = "\U00002705"
            no_emoji = "\U0001F6AB"
            await response_message.add_reaction(yes_emoji)
            await response_message.add_reaction(no_emoji)

    async def stats_autocomplete(
        self,
        interaction: discord.Interaction,
        current: str,
    ) -> List[app_commands.Choice[str]]:
        players = self.yahoo_api.get_players(
            current, guild_id=interaction.guild_id
        )
        if players:
            options = list(
                map(
                    lambda x: app_commands.Choice(
                        name=x["name"]["full"],
                        value=x["name"]["full"],
                    ),
                    players,
                )
            )
        else:
            options = []
        return options

    @app_commands.command(
        name="stats", description="Returns the details of the given player"
    )
    @app_commands.autocomplete(player_name=stats_autocomplete)
    async def stats(self, interaction: discord.Interaction, player_name: str):
        logger.info(
            "Command:Stats called in %i with player_name:%s",
            interaction.guild_id,
            player_name,
        )
        player = self.yahoo_api.get_player_details(
            player_name, guild_id=interaction.guild_id
        )
        if player:
            embed = self.get_player_embed(player)
            await interaction.response.send_message(embed=embed)
        else:
            await interaction.response.send_message("Player not found")

    def get_player_embed(self, player):
        embed = discord.Embed(
            title=player["name"]["full"],
            description="#" + player["uniform_number"],
            color=0xEEE657,
        )
        embed.add_field(name="Postion", value=player["primary_position"])
        embed.add_field(name="Team", value=player["editorial_team_abbr"])
        if "bye_weeks" in player:
            embed.add_field(name="Bye", value=player["bye_weeks"]["week"])
        if "player_points" in player:
            embed.add_field(
                name="Total Points", value=player["player_points"]["total"]
            )
        embed.add_field(name="Owner", value=player["owner"])
        embed.set_thumbnail(url=player["image_url"])
        return embed

    def get_player_text(self, player):
        player_details_text = (
            player["name"]["full"] + " #" + player["uniform_number"] + "\n"
        )
        player_details_text = (
            player_details_text
            + "Position: "
            + player["primary_position"]
            + "\n"
        )
        player_details_text = (
            player_details_text
            + "Team: "
            + player["editorial_team_abbr"]
            + "\n"
        )
        if "bye_weeks" in player:
            player_details_text = (
                player_details_text
                + "Bye: "
                + player["bye_weeks"]["week"]
                + "\n"
            )
        if "player_points" in player:
            player_details_text = (
                player_details_text
                + "Total Points: "
                + player["player_points"]["total"]
                + "\n"
            )
        player_owner = self.yahoo_api.get_player_owner(player["player_id"])
        if player_owner:
            player_details_text = (
                player_details_text + "Owner: " + player_owner
            )
        return player_details_text

    @app_commands.command(
        name="matchups", description="Returns the current weeks matchups"
    )
    async def matchups(
        self, interaction: discord.Interaction, week: Optional[int] = None
    ):
        logger.info(
            "Command:Matchups called in {} with week: {}".format(
                interaction.guild_id, week
            )
        )

        week, details = self.yahoo_api.get_matchups(
            guild_id=interaction.guild_id, week=week
        )
        if details:
            embed = discord.Embed(
                title="Matchups for Week {}".format(week),
                description="",
                color=0xEEE657,
            )
            for detail in details:
                embed.add_field(
                    name=detail["name"], value=detail["value"], inline=False
                )
            await interaction.response.send_message(embed=embed)
        else:
            await interaction.response.send_message(self.error_message)

    @app_commands.command(
        name="waivers",
        description="Returns the waiver transactions from the last 24 hours",
    )
    async def waivers(self, interaction: discord.Interaction, days: int = 1):
        logger.info("Command:Waivers called in %i", interaction.guild_id)

        await interaction.response.defer()
        embed_functions_dict = {
            "add/drop": utils.create_add_drop_embed,
            "add": utils.create_add_embed,
            "drop": utils.create_drop_embed,
        }
        ts = datetime.now() - timedelta(days=days)
        transactions = self.yahoo_api.get_transactions(
            guild_id=interaction.guild_id, timestamp=ts.timestamp()
        )
        if transactions:
            for transaction in transactions:
                await interaction.followup.send(
                    embed=embed_functions_dict[transaction["type"]](
                        transaction
                    )
                )
        else:
            await interaction.followup.send("No transactions found")



from discord.ext import commands
from discord import app_commands

import discord
import logging

from harambot.ui.views import ConfigView, ReportConfigView
from harambot.database.models import Guild


logger = logging.getLogger("discord.harambot.cogs.meta")
logger.setLevel(logging.INFO)


class Meta(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @app_commands.command(name="help", description="View available commands")
    async def help(self, interaction: discord.Interaction):
        embed = discord.Embed(
            title="Harambot",
            description="Yahoo Fantasy Sports Bot for Discord",
            color=0xEEE657,
        )
        embed.add_field(
            name="/ping", value="Gives the latency of harambot", inline=False
        )
        embed.add_field(
            name="/rip", value="Pay respects to Harambe", inline=False
        )
        embed.add_field(
            name="/standings",
            value="Returns the current standings of your league",
            inline=False,
        )
        embed.add_field(
            name="/roster team_name",
            value="Returns the roster of the given team",
            inline=False,
        )
        embed.add_field(
            name="/stats player_name",
            value="Returns the details of the given player",
            inline=False,
        )
        embed.add_field(
            name="/trade",
            value="Create poll for latest trade for league approval",
            inline=False,
        )
        embed.add_field(
            name="/matchups",
            value="Returns the current weeks matchups",
            inline=False,
        )
        embed.add_field(
            name="/waivers days",
            value="Returns the waiver wire transactions for the previous number of days",
            inline=False,
        )
        embed.add_field(
            name="/configure",
            value="Configure your guild for Harambot",
            inline=False,
        )
        await interaction.response.send_message(embed=embed)

    @app_commands.command(
        name="ping", description="Gives the latency of harambot"
    )
    async def ping(self, interaction: discord.Interaction):
        await interaction.response.send_message(self.bot.latency)

    @app_commands.command(
        name="configure",
        description="Configure your guild for Harambot",
    )
    @app_commands.checks.has_permissions(administrator=True)
    async def configure(self, interaction: discord.Interaction):
        await interaction.response.send_message(
            """
            Lets setup your guild
            1. Login into Yahoo and copy you authentication token
2. Configure harambot with your league information
            """,
            view=ConfigView(),
            ephemeral=True,
        )

    @configure.error
    async def configure_check_error(
        self, interaction: discord.Interaction, error
    ):
        if isinstance(error, app_commands.MissingPermissions):
            await interaction.response.send_message(
                "You do not have the required permissions to run this command."
            )

    def webhook_permissions(interaction: discord.Interaction):
        return interaction.guild.me.guild_permissions.manage_webhooks

    def guild_is_configured(interaction: discord.Interaction):
        return (
            Guild.select()
            .where(Guild.guild_id == str(interaction.guild_id))
            .exists()
        )

    @app_commands.command(
        name="reports",
        description="Configure automatic transaction and matchup reporting",
    )
    @app_commands.check(webhook_permissions)
    @app_commands.check(guild_is_configured)
    async def reports(
        self,
        interaction: discord.Interaction,
    ):
        message = "Set what channel transaction reports should be sent to."
        await interaction.response.send_message(
            message, view=ReportConfigView(), ephemeral=True
        )

    @reports.error
    async def reports_check_error(
        self, interaction: discord.Interaction, error
    ):
        await interaction.response.send_message(
            "Grant Harambot the Manage Webhooks permission to use this command"
        )



import logging
import discord
import queue

from logging.handlers import QueueHandler, QueueListener
from discord.ext import commands

from harambot.cogs.meta import Meta
from harambot.cogs.misc import Misc
from harambot.cogs.yahoo import YahooCog
from harambot.cogs.webserver import WebServer
from harambot.config import settings
from harambot.database.models import Guild
from harambot.database.migrations import migrations


que = queue.Queue(-1)  # no limit on size
queue_handler = QueueHandler(que)
handler = logging.StreamHandler()
listener = QueueListener(que, handler)
listener.start()

logger = logging.getLogger("discord.harambot")
if "LOGLEVEL" in settings:
    logger.setLevel(settings.LOGLEVEL)
else:
    logger.setLevel("DEBUG")


intents = discord.Intents.default()


bot = commands.Bot(
    command_prefix="$",
    description="",
    intents=intents,
)
bot.remove_command("help")


@bot.event
async def on_ready():
    await bot.add_cog(Meta(bot))
    await bot.add_cog(YahooCog(bot, settings.yahoo_key, settings.yahoo_secret))
    await bot.add_cog(Misc(bot))
    server = WebServer(bot)
    await bot.add_cog(server)
    bot.loop.create_task(server.webserver())

    if not Guild.table_exists():
        Guild.create_table()
    if "RUN_MIGRATIONS" in settings and settings.run_migrations:
        migrations[settings.version]()
    await bot.tree.sync()
    logger.info("Everything's all ready to go~")


@bot.event
async def on_guild_join(guild):
    logger.info("Joined {}".format(guild.name))
    if not Guild.select().where(Guild.guild_id == str(guild.id)).exists():
        logger.info("Guild not configured!")


def run():
    bot.run(settings.discord_token, reconnect=True, log_handler=queue_handler)


run()



import logging
import time

from discord import SyncWebhook
from discord.errors import NotFound

from harambot.config import settings
from harambot.database.models import Guild
from harambot.yahoo_api import Yahoo
from harambot import utils


from datetime import datetime, timedelta

logging.basicConfig()
logger = logging.getLogger("harambot.transaction_polling")
if "LOGLEVEL" in settings:
    logger.setLevel(settings.LOGLEVEL)
    logging.getLogger("discord").setLevel(settings.LOGLEVEL)
    logging.getLogger("peewee").setLevel(settings.LOGLEVEL)
else:
    logger.setLevel("DEBUG")
    logging.getLogger("discord").setLevel("DEBUG")
    logging.getLogger("peewee").setLevel("DEBUG")

logging.getLogger("yahoo_oauth").disabled = True


embed_functions_dict = {
    "add/drop": utils.create_add_drop_embed,
    "add": utils.create_add_embed,
    "drop": utils.create_drop_embed,
}


def poll_transactions(guild: Guild):
    logger.info("Polling transactions for {}".format(guild.guild_id))
    YahooAPI = Yahoo()
    ts = datetime.now() - timedelta(minutes=5)
    try:
        transactions = YahooAPI.get_transactions(
            timestamp=ts.timestamp(), guild_id=guild.guild_id
        )
        logger.info(
            "Found {} transactions for guild {}".format(
                str(len(transactions)), guild.guild_id
            )
        )
        if transactions:

            for transaction in transactions:
                embed = embed_functions_dict[transaction["type"]](transaction)
                webhook = SyncWebhook.from_url(
                    guild.transaction_polling_webhook
                )
                try:
                    webhook.send(embed=embed)
                except NotFound:
                    logger.exception(
                        "Webhook not found for {}".format(guild.guild_id)
                    )
                time.sleep(
                    1
                )  # sleep for 1 second between transactions to avoid rate limits
    except Exception:
        logger.info(
            "Error fetching transactions for {}".format(guild.guild_id)
        )


def report_service():
    logger.info("Starting transaction polling service")
    for guild in Guild.select().where(
        Guild.transaction_polling_service_enabled == 1
    ):
        poll_transactions(guild=guild)
        time.sleep(
            5
        )  # sleep for 5 seconds between each guild to avoid rate limits


if __name__ == "__main__":
    logger.info("Running Report Service")
    report_service()
    logger.info("Report Service Finished")



import discord
import logging

from discord.utils import MISSING
from typing import Optional
from harambot.database.models import Guild
from harambot.utils import yahoo_auth

logger = logging.getLogger(__file__)
logger.setLevel(logging.INFO)


class ConfigModal(discord.ui.Modal, title="Configure Guild"):

    yahoo_token = discord.ui.TextInput(
        label="Yahoo Token",
        placeholder="Enter the token from the Yahoo login link",
    )

    league_id = discord.ui.TextInput(
        label="Yahoo League ID", placeholder="Enter Yahoo League ID"
    )
    league_type = discord.ui.TextInput(
        label="Yahoo League Type",
        placeholder="Enter Yahoo League Type(nfl, nhl, nba, mlb)",
    )
    RIP_text = discord.ui.TextInput(
        label="RIP command text",
        placeholder="Enter text to use with $RIP command",
    )
    RIP_image_url = discord.ui.TextInput(
        label="RIP Image",
        placeholder="Enter comma separated list of urls for $RIP command",
    )

    guild = None
    view = None

    def __init__(
        self,
        *,
        title: str = MISSING,
        timeout: Optional[float] = None,
        custom_id: str = MISSING,
        guild_id: str = None,
        view: discord.ui.View = None,
    ) -> None:
        super().__init__(title=title, timeout=timeout, custom_id=custom_id)
        self.view = view
        self.guild = Guild.get_or_none(Guild.guild_id == str(guild_id))
        if self.guild:
            self.remove_item(self.yahoo_token)
            self.league_id.default = self.guild.league_id
            self.league_type.default = self.guild.league_type
            self.RIP_text.default = self.guild.RIP_text
            self.RIP_image_url.default = self.guild.RIP_image_url

    async def on_submit(self, interaction: discord.Interaction):
        details = {
            "league_id": self.league_id.value,
            "league_type": self.league_type.value,
            "RIP_text": self.RIP_text.value,
            "RIP_image_url": self.RIP_image_url.value,
        }
        if self.guild:
            Guild.update(details).where(
                Guild.guild_id == self.guild.guild_id
            ).execute()
        else:
            oauth_details = yahoo_auth(self.yahoo_token.value)
            if not oauth_details:
                await interaction.response.send_message(
                    "Failed to authenticate with Yahoo API.\
                     Please try again",
                    ephemeral=True,
                )
            else:
                details.update(oauth_details)
                self.guild = Guild(
                    guild_id=str(interaction.guild_id), **details
                )
                self.guild.save()
        await interaction.response.send_message(
            "Guild settings updated!",
            ephemeral=True,
        )
        self.view.stop()

    async def on_error(
        self, interaction: discord.Interaction, error: Exception
    ) -> None:
        logger.exception(error)
        await interaction.response.send_message(
            "Oops! Something went wrong with configuring your guild. Please try again.",
            ephemeral=True,
        )



import logging

from peewee import SqliteDatabase
from peewee import Model
from peewee import TextField, IntegerField, BigIntegerField, TimestampField
from playhouse.db_url import connect
from harambot.config import settings
from harambot.database.fields import EncryptedField

logger = logging.getLogger("peewee")
if "LOGLEVEL" in settings:
    logger.setLevel(settings.LOGLEVEL)
else:
    logger.setLevel("DEBUG")

if "DATABASE_URL" in settings:
    database = connect(settings.database_url)
else:
    logger.info("Using in-memory database")
    database = SqliteDatabase(":memory:")

KEY = ""
if "HARAMBOT_KEY" in settings:
    KEY = settings.HARAMBOT_KEY
else:
    from cryptography.fernet import Fernet

    fernet_key = Fernet.generate_key()
    KEY = fernet_key.decode()


class BaseModel(Model):
    class Meta:
        database = database


class Guild(BaseModel):
    guild_id = TextField(unique=True)
    access_token = EncryptedField(key=KEY)
    refresh_token = EncryptedField(key=KEY)
    expires_in = IntegerField()
    token_type = TextField()
    xoauth_yahoo_guid = TextField(null=True)
    token_time = BigIntegerField()
    league_id = TextField()
    league_type = TextField()
    RIP_text = TextField()
    RIP_image_url = TextField()
    transaction_polling_service_enabled = IntegerField(default=0)
    transaction_polling_webhook = TextField(null=True)
    last_transaction_check = TimestampField()



import logging

from playhouse.migrate import SqliteMigrator, MySQLMigrator, PostgresqlMigrator
from playhouse.migrate import migrate
from playhouse.dataset import DataSet
from peewee import PostgresqlDatabase, MySQLDatabase, SqliteDatabase
from peewee import TimestampField, IntegerField, TextField
from playhouse.db_url import connect
from cryptography.fernet import Fernet


from harambot.config import settings

logger = logging.getLogger("discord")
logger.setLevel(settings.LOGLEVEL)


if "DATABASE_URL" in settings:
    database = connect(settings.database_url)
else:
    database = SqliteDatabase(":memory:")

# Get migrator
migrator = None
if isinstance(database, MySQLDatabase):
    migrator = MySQLMigrator(database)
elif isinstance(database, PostgresqlDatabase):
    migrator = PostgresqlMigrator(database)
else:
    migrator = SqliteMigrator(database)


# Migration Functions
def beta003_migrations():
    last_transaction_check = TimestampField()
    migrate(
        migrator.add_column(
            "guild", "last_transaction_check", last_transaction_check
        )
    )


def beta040_migrations():
    # Encrypt token fields
    f = Fernet(settings.HARAMBOT_KEY)
    if "DATABASE_URL" in settings:
        dataSet = DataSet(settings.database_url)
    else:
        dataSet = DataSet(":memory:")
    guilds = dataSet["guild"]
    for guild in guilds.all():
        logger.info(
            "Encrypted token: "
            + "'{}'".format(f.encrypt(guild["access_token"].encode()).decode())
        )
        logger.info("Guild ID:")
        logger.info(guild["id"])
        with dataSet.transaction() as txn:
            guilds.update(
                id=guild["id"],
                access_token=f.encrypt(
                    guild["access_token"].encode()
                ).decode(),
                columns=["id"],
            )
            guilds.update(
                id=guild["id"],
                refresh_token=f.encrypt(
                    guild["refresh_token"].encode()
                ).decode(),
                columns=["id"],
            )
            txn.commit()
    # Add new fields for transaction polling
    transaction_polling_service_enabled = IntegerField(default=0)
    transaction_polling_webhook = TextField(null=True)
    migrate(
        migrator.add_column(
            "guild",
            "transaction_polling_service_enabled",
            transaction_polling_service_enabled,
        ),
        migrator.add_column(
            "guild",
            "transaction_polling_webhook",
            transaction_polling_webhook,
        ),
    )


# Migration dictionary
migrations = {}
migrations["0.0.3-Beta"] = beta003_migrations
migrations["0.4.0-Beta"] = beta040_migrations



import discord
import logging
import random

from discord.ext import commands
from discord import app_commands
from typing import Optional
from harambot.database.models import Guild

logger = logging.getLogger("discord.harambot.cogs.misc")
logger.setLevel(logging.INFO)


class Misc(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @app_commands.command(name="rip", description="Pay respects to Harambe")
    @app_commands.describe(deceased="Who you are paying respects to")
    async def rip(
        self, interaction: discord.Interaction, deceased: Optional[str] = None
    ):

        logger.info("RIP called")
        guild = Guild.get(Guild.guild_id == str(interaction.guild_id))
        message = guild.RIP_text + " " + (deceased if deceased else "Harambe")
        embed = discord.Embed(title="", description="", color=0xEEE657)
        if "," in guild.RIP_image_url:
            images = guild.RIP_image_url.split(",")
            guild.RIP_image_url = random.choice(images)
        embed.set_image(url=guild.RIP_image_url)
        await interaction.response.send_message(content=message, embed=embed)



import logging
import os
import objectpath
import functools

from yahoo_fantasy_api import game
from cachetools import cached, keys, TTLCache
from playhouse.shortcuts import model_to_dict
from yahoo_oauth import OAuth2

from harambot.database.models import Guild
from harambot.config import settings

logger = logging.getLogger("discord.harambot.yahoo_api")


dir_path = os.path.dirname(os.path.realpath(__file__))
cache = TTLCache(maxsize=1024, ttl=600)


class Yahoo:

    scoring_type = None
    league_id = None
    league_type = None

    def handle_oauth(f):
        @functools.wraps(f)
        def wrapper(self, *args, **kwargs):
            guild_id = kwargs.get("guild_id")
            guild = Guild.get_or_none(Guild.guild_id == str(guild_id))
            if guild is None:
                logger.error(
                    "Guild with id %i does not exist in the database",
                    guild_id,
                )
                return None
            self.oauth = OAuth2(
                settings.yahoo_key,
                settings.yahoo_secret,
                store_file=False,
                **model_to_dict(guild),
            )
            self.league_id = guild.league_id
            self.league_type = guild.league_type
            return f(self, *args, **kwargs)

        return wrapper

    def league(self):
        try:
            if not self.oauth.token_is_valid():
                self.oauth.refresh_access_token()
        except Exception:
            logger.exception(
                "Error while refreshing access token for league: {}".format(
                    self.league_id
                )
            )
            return None

        try:
            gm = game.Game(self.oauth, self.league_type)
            try:
                for id in gm.league_ids():
                    if self.league_id in id:
                        self.league_id = id
                        break
                league = gm.to_league(self.league_id)
                self.scoring_type = league.settings()["scoring_type"]
                return league
            except (RuntimeError, AssertionError):
                logger.error(
                    "Error fetching league ids from Yahoo, trying with constructed league id"
                )
                league = gm.to_league(gm.game_id() + ".l." + self.league_id)
                self.scoring_type = league.settings()["scoring_type"]
                return league
        except Exception:
            logger.exception(
                "Error while fetching league details for league {}".format(
                    self.league_id
                )
            )
            return None

    @cached(cache, key=functools.partial(keys.hashkey, "get_teams"))
    @handle_oauth
    def get_teams(self, guild_id):
        try:
            return self.league().teams()
        except Exception:
            logger.exception(
                "Error while fetching teams for league {} in guild {}".format(
                    self.league_id,
                    guild_id,
                )
            )
            return None

    @cached(cache, key=functools.partial(keys.hashkey, "get_players"))
    @handle_oauth
    def get_players(self, player, guild_id):
        try:
            return self.league().player_details(player)
        except Exception:
            logger.exception(
                "Error while fetching players for league {} in guild: {}".format(
                    self.league_id,
                    guild_id,
                )
            )
            return None

    @cached(cache, key=functools.partial(keys.hashkey, "get_standings"))
    @handle_oauth
    def get_standings(self, guild_id):
        try:
            standings = []
            for idx, team in enumerate(self.league().standings()):
                outcomes = team["outcome_totals"]
                record = "{}-{}-{}".format(
                    outcomes["wins"], outcomes["losses"], outcomes["ties"]
                )
                standings.append(
                    {
                        "place": str(idx + 1) + ". " + team["name"],
                        "record": record,
                    }
                )
            return standings
        except Exception:
            logger.exception(
                "Error while fetching standings for league {} in guild {}".format(
                    self.league_id,
                    guild_id,
                )
            )
            return None

    @cached(cache)
    @handle_oauth
    def get_roster(self, team_name, guild_id):
        try:
            team_details = self.league().get_team(team_name)
            if team_details:
                return team_details[team_name].roster(
                    self.league().current_week()
                )
            else:
                return None
        except Exception:
            logger.exception(
                "Error while fetching roster for team: {} in league {} for guild: {}".format(
                    team_name, self.league_id, guild_id
                )
            )
            return None

    @cached(cache)
    @handle_oauth
    def get_player_details(self, player_name, guild_id):
        try:
            player = self.league().player_details(player_name)[0]
            player["owner"] = self.get_player_owner(player["player_id"])
            return player
        except Exception:
            logger.exception(
                "Error while fetching player details for player: \
                    {} in league {} in guild {}".format(
                    player_name, self.league_id, guild_id
                )
            )
            return None

    def get_player_owner(self, player_id):
        try:
            player_ownership = self.league().ownership([player_id])[
                str(player_id)
            ]
            if "owner_team_name" in player_ownership:
                return player_ownership["owner_team_name"]
            else:
                ownership_map = {
                    "freeagents": "Free Agent",
                    "waivers": "On Waivers",
                }
                return ownership_map.get(
                    player_ownership["ownership_type"], ""
                )
        except Exception:
            logger.exception(
                "Error while fetching ownership for player id: \
                    {} in league {}".format(
                    player_id, self.league_id
                )
            )
            return None

    @cached(cache)
    @handle_oauth
    def get_matchups(self, guild_id, week=None):
        try:
            if not week:
                week = self.league().current_week()
            matchups = objectpath.Tree(
                self.league().matchups(week=week)
            ).execute("$..scoreboard..matchups..matchup..teams")

            details = []
            divider = "--------------------------------------"
            for matchup in matchups:
                team1_details = self.get_matchup_details(matchup["0"]["team"])
                team2_details = self.get_matchup_details(matchup["1"]["team"])
                details.append(
                    {
                        "name": "{} vs {}".format(
                            team1_details["name"], team2_details["name"]
                        ),
                        "value": team1_details["text"]
                        + team2_details["text"]
                        + divider,
                    }
                )
            return str(week), details
        except Exception as e:
            logger.exception(
                "Error while fetching matchups for league: {} for guild: {}".format(
                    self.league_id,
                    guild_id,
                ),
                e,
            )

    def get_matchup_details(self, team):
        team_name = team[0][2]["name"]
        team_details = "***{}*** \n".format(team_name)

        actual_points = team[1]["team_points"]["total"]
        team_details += "Score: {} \n".format(actual_points)

        if "team_projected_points" in team[1]:
            projected_points = team[1]["team_projected_points"]["total"]
            team_details += "Projected Score: {} \n".format(projected_points)

        if "win_probability" in team[1]:
            win_probability = "{:.0%}".format(team[1]["win_probability"])
            team_details += "Win Probability: {} \n".format(win_probability)

        if "team_remaining_games" in team[1]:
            remaining_games = team[1]["team_remaining_games"]["total"][
                "remaining_games"
            ]
            live_games = team[1]["team_remaining_games"]["total"]["live_games"]
            completed_games = team[1]["team_remaining_games"]["total"][
                "completed_games"
            ]
            team_details += "Remaining Games: {} \n".format(remaining_games)
            team_details += "Live Games: {} \n".format(live_games)
            team_details += "Completed Games: {} \n".format(completed_games)

        return {"name": team_name, "text": team_details}

    @cached(cache, key=functools.partial(keys.hashkey, "get_latest_trade"))
    @handle_oauth
    def get_latest_trade(self, guild_id):
        try:
            for key, values in self.league().teams().items():
                if "is_owned_by_current_login" in values:
                    team = self.league().to_team(key)
                    accepted_trades = list(
                        filter(
                            lambda d: d["status"] == "accepted",
                            team.proposed_trades(),
                        )
                    )
                    if accepted_trades:
                        return accepted_trades[0]
            return None
        except Exception:
            logger.exception(
                "Error fetching latest trades for league: {} in guild {}".format(
                    self.league_id, guild_id
                )
            )
            return None

    @cached(cache, key=functools.partial(keys.hashkey, "get_transactions"))
    @handle_oauth
    def get_transactions(self, guild_id, timestamp=0.0):
        try:
            transactions = self.league().transactions("add,drop", "")
            filtered_transactions = [
                t for t in transactions if float(t["timestamp"]) > timestamp
            ]
            return filtered_transactions
        except Exception:
            logger.exception(
                "Error fetching latest waivers for league: {} in guild {}".format(
                    self.league_id,
                    guild_id,
                )
            )
            return []



from cryptography.fernet import Fernet
from peewee import Field


class EncryptedField(Field):
    def __init__(self, key, *args, **kwargs):
        super(EncryptedField, self).__init__(*args, **kwargs)
        self.key = key

    def db_value(self, value):
        f = Fernet(self.key)
        return f.encrypt(value.encode()).decode()

    def python_value(self, value):
        f = Fernet(self.key)
        return f.decrypt(value.encode()).decode()



from aiohttp import web
from discord.ext import commands
from yahoo_fantasy_api import oauth2_logger
from harambot.config import settings

import logging

oauth2_logger.cleanup()

logger = logging.getLogger("discord.harambot.cogs.webserver")
logger.setLevel(logging.INFO)


class WebServer(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    async def webserver(self):
        async def handler(request):
            status = f"""
            Harambot
            Harambot v{settings.version} is running!
            Bot status: {request.config_dict["bot"].status}
            Latency: {round(request.config_dict["bot"].latency * 1000)}ms
            """
            return web.Response(text=status)

        app = web.Application()
        app.router.add_get("/", handler)
        app["bot"] = self.bot
        runner = web.AppRunner(app)
        await runner.setup()
        if "PORT" in settings:
            site = web.TCPSite(runner, "0.0.0.0", settings.port)
        else:
            site = web.TCPSite(runner, "0.0.0.0", 10000)
        await self.bot.wait_until_ready()
        await site.start()
        logger.info("Webserver started on port {}".format(settings.port))
